use crate::models::ItemStatus;
use crate::FeverApi;
use reqwest::Client;
use std::env;
use url::Url;

fn test_setup_env() -> FeverApi {
    dotenv::dotenv().expect("Failed to read .env file");
    let url = env::var("FEVER_URL").expect("Failed to read FEVER_URL");
    let user = env::var("FEVER_USER").expect("Failed to read FEVER_USER");
    let pw = env::var("FEVER_PW").expect("Failed to read FEVER_PW");

    let url = Url::parse(&url).unwrap();
    FeverApi::new(&url, &user, &pw)
}

#[tokio::test(basic_scheduler)]
async fn api_version() {
    let api = test_setup_env();

    let response = api.get_api_version(&Client::new()).await;
    assert!(response.is_ok());
    assert!(response.unwrap() == 3);
}

#[tokio::test(basic_scheduler)]
async fn valid_credentials() {
    let api = test_setup_env();

    let response = api.valid_credentials(&Client::new()).await;
    assert!(response.is_ok());
    assert!(response.unwrap());
}

#[tokio::test(basic_scheduler)]
async fn groups() {
    let api = test_setup_env();

    let response = api.get_groups(&Client::new()).await;
    assert!(response.is_ok());
}

#[tokio::test(basic_scheduler)]
async fn feeds() {
    let api = test_setup_env();

    let response = api.get_feeds(&Client::new()).await;
    assert!(response.is_ok());
}

#[tokio::test(basic_scheduler)]
async fn favicons() {
    let api = test_setup_env();

    let response = api.get_favicons(&Client::new()).await;
    assert!(response.is_ok());
}

#[tokio::test(basic_scheduler)]
async fn items() {
    let api = test_setup_env();

    let response = api.get_items(&Client::new()).await;
    assert!(response.is_ok());

    let response = api.get_items_since(5, &Client::new()).await;
    assert!(response.is_ok());

    let response = api.get_items_max(22, &Client::new()).await;
    assert!(response.is_ok());

    let response = api.get_items_with(vec![5, 7, 8, 9, 10], &Client::new()).await;
    assert!(response.is_ok());
}

#[tokio::test(basic_scheduler)]
async fn links() {
    let api = test_setup_env();

    let response = api.get_links(&Client::new()).await;
    assert!(response.is_ok());
    
    let response = api.get_links_with(0, 7, 1, &Client::new()).await;
    assert!(response.is_ok());
}

#[tokio::test(basic_scheduler)]
async fn unread() {
    let api = test_setup_env();

    let response = api.get_unread_items(&Client::new()).await;
    assert!(response.is_ok());
}

#[tokio::test(basic_scheduler)]
async fn saved() {
    let api = test_setup_env();

    let response = api.get_saved_items(&Client::new()).await;
    assert!(response.is_ok());
}

#[tokio::test(basic_scheduler)]
async fn mark_item() {
    let api = test_setup_env();

    let response = api.get_items_since(1, &Client::new()).await;
    assert!(response.is_ok());
    let valid_item = &response.unwrap().items[0];
    let id = valid_item.id;

    let response = api.get_saved_items(&Client::new()).await;
    assert!(response.is_ok());
    let tmp: usize = response.unwrap().saved_item_ids.len();

    let response = api.mark_item(ItemStatus::Saved, id, &Client::new()).await;
    assert!(response.is_ok());

    let response = api.get_saved_items(&Client::new()).await;
    assert!(response.is_ok());
    assert_eq!(tmp + 1, response.unwrap().saved_item_ids.len());

    let response = api.mark_item(ItemStatus::Unsaved, id, &Client::new()).await;
    assert!(response.is_ok());

    let response = api.get_saved_items(&Client::new()).await;
    assert!(response.is_ok());
    assert_eq!(tmp, response.unwrap().saved_item_ids.len());
}